import tensorflow as tf
import cv2
import numpy as np
import requests

# TensorFlow Hubから事前学習済みモデルをロード
model_url = "https://tfhub.dev/tensorflow/ssd_mobilenet_v2/2"
detector = tf.saved_model.load(model_url)

# モデルの推論関数
def run_inference_for_single_image(model, image):
    # 画像をモデルに入力できる形式に変換
    image = np.asarray(image)
    image = np.expand_dims(image, axis=0)
    image = tf.convert_to_tensor(image, dtype=tf.float32)
    
    # 推論を実行
    model_fn = model.signatures['serving_default']
    output_dict = model_fn(image)
    
    # 推論結果の取得
    return {key:value.numpy() for key,value in output_dict.items()}

# ESP32-CAMからの映像ストリームURL
url = "http://192.168.3.39/stream"
cap = cv2.VideoCapture(url)

while True:
    ret, frame = cap.read()
    if not ret:
        print("Failed to retrieve frame")
        break

    # 画像サイズの調整（モデルに合わせたサイズにリサイズ）
    image_resized = cv2.resize(frame, (300, 300))
    
    # 障害物認識を実行
    output_dict = run_inference_for_single_image(detector, image_resized)
    
    # 結果を処理し、障害物を検出
    num_detections = int(output_dict['num_detections'])
    for i in range(num_detections):
        class_id = int(output_dict['detection_classes'][0][i])
        score = float(output_dict['detection_scores'][0][i])
        
        if score > 0.5:  # 信頼度が50%以上の物体を障害物として検出
            # 物体の位置を取得
            ymin, xmin, ymax, xmax = output_dict['detection_boxes'][0][i]
            (h, w) = frame.shape[:2]
            (xmin, xmax, ymin, ymax) = (xmin * w, xmax * w, ymin * h, ymax * h)
            
            # 障害物が検出された領域を四角で囲む
            cv2.rectangle(frame, (int(xmin), int(ymin)), (int(xmax), int(ymax)), (0, 255, 0), 2)
            cv2.putText(frame, f"Object Detected: {score:.2f}", (int(xmin), int(ymin)-10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (0, 255, 0), 2)

    # 結果を表示
    cv2.imshow("Frame", frame)

    # 'q'キーで終了
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

cap.release()
cv2.destroyAllWindows()
